import logging
import uuid

__all__ = ["randomize_field", "randomize_sub_tree_field"]

from typing import Collection, Union, Any

logger = logging.getLogger(__name__)


def randomize_sub_tree_field(selected_nodes: Union[Any, Collection[Any]], field_name: str) -> None:
  # noinspection PyUnresolvedReferences
  from qgis.core import QgsLayerTreeGroup, QgsLayerTreeLayer, QgsLayerTreeNode

  if isinstance(selected_nodes, QgsLayerTreeLayer):
    randomize_field(selected_nodes, field_name=field_name)
  elif isinstance(selected_nodes, QgsLayerTreeGroup):
    randomize_sub_tree_field(selected_nodes.children(), field_name=field_name)
  elif isinstance(selected_nodes, QgsLayerTreeNode):
    if selected_nodes.nodeType() == QgsLayerTreeNode.NodeGroup:
      randomize_sub_tree_field(selected_nodes.children(), field_name=field_name)
    else:
      logger.error(f'Node {selected_nodes} was not supported in randomize_sub_tree_field, skipping')
  else:
    if len(selected_nodes) == 0:
      logger.error(
        f"'Number of selected nodes was {len(selected_nodes)}, please supply some"
      )
      return

    for group in iter(selected_nodes):
      if isinstance(group, QgsLayerTreeLayer):
        randomize_field(group, field_name=field_name)
      elif isinstance(group, QgsLayerTreeGroup):
        randomize_sub_tree_field(group.children(), field_name=field_name)
      elif isinstance(group, QgsLayerTreeNode):
        if group.nodeType() == QgsLayerTreeNode.NodeGroup:
          randomize_sub_tree_field(group.children(), field_name=field_name)
        else:
          logger.error(f'Node {group} was not supported in randomize_sub_tree_field, skipping')
      else:
        logger.error(f'Node {group} was not supported in randomize_sub_tree_field, skipping')


def randomize_field(tree_layer: Any, field_name: str) -> None:  #: QgsLayerTreeLayer
  """
      https://qgis.org/pyqgis/3.28/core/QgsVectorLayer.html#qgis.core.QgsVectorLayer.changeAttributeValues

  changeAttributeValues
  fid: int, newValues: Dict[int, Any], oldValues: Dict[int, Any] = {}, skipDefaultValues: bool = False

      :param field_name:
      :param tree_layer:
      :return:
  """
  if tree_layer is None:
    logger.error(f"Tree layer was None")
    return
  # logger.info(f'Randomizing {field_name} in {tree_layer.layer().name()}')

  layer = tree_layer.layer()

  field_idx = layer.fields().indexFromName(field_name)

  if field_idx >= 0:
    layer.startEditing()
    # layer.beginEditCommand(f"Regenerate {field_name}")
    logger.info(
      f"Randomizing {field_name}:{field_idx} in {tree_layer.layer().name()}"
    )

    for i in range(layer.featureCount() + 1):
      layer.changeAttributeValue(i, field_idx, uuid.uuid4().hex)

    # layer.rollBack()
    # layer.endEditCommand()
    layer.commitChanges()
  else:
    logger.error(f"Did not find {field_name} in {layer.name()}")



import logging
from typing import Optional, Any, Union

# noinspection PyUnresolvedReferences
from qgis.core import QgsLayerTreeGroup, QgsLayerTreeLayer, QgsProject
# noinspection PyUnresolvedReferences
from qgis.utils import iface

__all__ = ["duplicate_groups", "select_layer_in_group", "is_group_selected"]

from jord.qgis_utilities.helpers.copying import deepcopy_layer

try:
  from types import EllipsisType
except ImportError:
  EllipsisType = type(...)

logger = logging.getLogger(__name__)


def duplicate_groups(
    group_to_duplicate,
    *,
    group_parent: Optional[Any] = None,
    new_name: Union[str, EllipsisType, None] = None,
) -> QgsLayerTreeGroup:
  if group_to_duplicate is None:
    logger.error("Group was None")
    return

  logger.info(f"Duplicating {group_to_duplicate.name()}")

  if new_name is ... or new_name == "" or new_name is None:
    new_name = f"{group_to_duplicate.name()} (Copy)"

  sub_items = []
  assert (
      group_to_duplicate is not None
  ), f"{group_to_duplicate=} is required to create a duplicate group"

  if group_parent is None:
    group_parent = group_to_duplicate.parent()

  if group_parent is None:
    raise ValueError(f"Group parent was not found for {group_to_duplicate}")

  new_group_parent = group_parent.addGroup(new_name)
  for original_group_child in group_to_duplicate.children():
    if isinstance(original_group_child, QgsLayerTreeGroup):
      new_sub_group, sub_sub_items = duplicate_groups(
        original_group_child, group_parent=new_group_parent, new_name=...
      )
      sub_items.extend([new_sub_group, *sub_sub_items])
    elif isinstance(original_group_child, QgsLayerTreeLayer):
      sub_items.append(
        duplicate_tree_node(new_group_parent, original_group_child)
      )
    else:
      logger.error(f"{original_group_child} no supported in duplication")

  return new_group_parent, sub_items


def duplicate_tree_node(new_group_parent, original_group_child):
  original_layer = original_group_child.layer()
  new_layer_copy = deepcopy_layer(original_layer)
  QgsProject.instance().addMapLayer(new_layer_copy, False)

  if False:
    new_layer_tree_node = QgsLayerTreeLayer(
      new_layer_copy
    )  # WORKS BUT MISSING STYLING
    new_group_parent.addChildNode(new_layer_tree_node)
  elif False:
    # Does not WORK EITHER
    new_layer_tree_node = QgsLayerTreeLayer(
      new_layer_copy.id(),
      original_layer.name(),
      new_layer_copy.source(),
      original_layer.providerType(),
    )
  elif False:
    # THIS DOES NOT WORK!!!
    new_layer_tree_node = (
      original_group_child.clone()
    )  # THIS JUST CREATES NEW VIEW of the same data!!!!
    new_layer_tree_node.layer().setDataSource(
      new_layer_copy.source(),
      new_layer_copy.name(),
      new_layer_copy.providerType(),
    )
  elif True:
    new_group_parent.insertLayer(0, new_layer_copy)
  else:
    raise Exception()

  return new_layer_copy


def select_layer_in_group(layer_name, group_name):
  group = QgsProject.instance().layerTreeRoot().findGroup(group_name)
  if group is not None:
    for child in group.children():
      if child.name() == layer_name:
        iface.setActiveLayer(child.layer())


def is_group_selected(group_name):
  group = QgsProject.instance().layerTreeRoot().findGroup(group_name)
  return group in iface.layerTreeView().selectedNodes()
